;******************************************************************************
; 			the lcd module subroutines                                     		 
;******************************************************************************
;	sendchar - sends character to lcd                                		
;	this routine splits the character into the upper and lower       
;	nibbles and sends them to the lcd, upper nibble first.           
;******************************************************************************
 
;******************************************************************************
; command sequence for 2 lines of 5x7 characters 4-bit operation
;******************************************************************************

Lcd_init
	bsf	rp0
	bcf	TRISA,1		; Set RA1 as output
	bcf	rp0
		bsf	KeyEnable	; Set for lcd use
	    	bsf     rp0    		; Select register page 1
	    	movlw   Portb_Out
	    	andwf   TRISB,f 	; Set for output to display
	   	bcf     rp0    		; Select register page 0

	    	call	lcd_delay

	    	movlw   b'00110000'     ; display in 8 bit mode.
	    	call 	LCD_Send	; Send data to LCD
	   	bsf     Disp_En 
	   	bcf     Disp_En
 
        	call    lcd_delay
 
cmd_seq
	    	movlw   b'00110000'     ; 8-bit high nibble xfer
	    	call 	LCD_Send	; Send data to LCD
	    	bsf     Disp_En		; and 8-bit modes
	    	bcf     Disp_En
		
		call	lcd_delay

 		movlw   b'00110000'     ; 8-bit high nibble xfer
	    	call 	LCD_Send	; Send data to LCD
	    	bsf    	Disp_En		; and 8-bit modes
	    	bcf     Disp_En

	    	clrf    Lsd             ; short delay
loop3       
		decfsz  Lsd, f
	    	goto    loop3
            		
		movlw   b'00100000'     ; 8-bit transfer turns registers to
	    	call 	LCD_Send	; Send data to LCD
	    	bsf     Disp_En   	; for every byte this point on
	    	bcf     Disp_En   

; busy flag should be valid after this point
 
		movlw   Func_set
		call    send_cmd
		movlw   Display_off
		call    send_cmd
		movlw   Display_on         
		call    send_cmd        
		movlw   Entry_inc       
		call    send_cmd   
		return

;******************************************************************************		
; this routine takes the calculated times that the delay loop needs to
; be executed, based on the Lcd_init_delay equate 
;******************************************************************************

lcd_delay   
		movlw   Lcd_init_delay  ;
	    	movwf   Msd             ;  use Msd and Lsd registers to 
	    	clrf    Lsd             ; initilize lcd
loop2       	decfsz  Lsd, f          ; delay time=Msd *((3 * 256) + 3)* tcy
	    	goto    loop2           ;
	    	clrwdt			; Clear watchdog timer
	    	decfsz  Msd, f          ;
end_lcd_delay
	    	goto    loop2           ;
		return
		;
send_char
	bsf	rp0
	bcf	TRISA,1		; Set RA1 as output
	bcf	rp0
	    	movwf   Char           	; character to be sent is in w
	    	call	busy_check      ; wait for lcd to be ready
        	movf   	Char, w          
	    	andlw   0xf0            ; get upper nibble
	    	call 	LCD_Send		; Send data to LCD
	    	bcf     R_w   		; set lcd to read
	    	bsf     Rs    		; set lcd to data mode
	    	bsf     Disp_En    	; toggle Disp_En for lcd
	    	bcf     Disp_En
	    	swapf   Char, w		; Swap nibbles leaving in w
	    	andlw   0xf0            ; get lower nibble (previously swapped)
	    	call 	LCD_Send		; Send data to LCD
	    	bsf     Disp_En    	; toggle E for lcd
 	    	bcf     Disp_En
		bcf	KeyEnable	; Set to read keys
	    	bsf     rp0    		; Select register page 1
	    	movlw   Portb_In 	; Set for display input
	    	iorwf   TRISB,f
	    	bcf     rp0    		; Select register page 0
	    	bsf	R_w		; ##########################
	bsf	rp0
	bsf	TRISA,1		; Set RA1 as input
	bcf	rp0
		return

;******************************************************************************
; 			send_cmd - sends command to lcd                                 
; 	this routine splits the command into the upper and lower        
; 	nibbles and sends them to the lcd, upper nibble first.          
;******************************************************************************

send_cmd
	bsf	rp0
	bcf	TRISA,1		; Set RA1 as output
	bcf	rp0
	    	movwf   Char          	; character to be sent is in w
	    	call    busy_check	; wait for lcd to be ready
	    	movf   	Char,w          
	    	andlw   0xf0      	; get upper nibble
	    	call 	LCD_Send		; Send data to LCD
	    	bcf     R_w    		; set lcd to read
	    	bcf     Rs     		; set lcd to command mode
;		movfw	Shadow_A
;		movwf	PORTA
	    	bsf     Disp_En     	; toggle E for lcd
	    	bcf     Disp_En
	    	swapf   Char,w
	    	andlw   0xf0            ; get lower nibble
	    	call 	LCD_Send	; Send data to LCD
	    	bsf     Disp_En     	; toggle E for lcd
	    	bcf     Disp_En
		bcf	KeyEnable
	    	bsf     rp0    		; Select register page 1
	    	movlw   Portb_In	; Set for display input
	    	iorwf   TRISB,f
	    	bcf     rp0    		; Select register page 0
	    	bsf	R_w	
	    	bcf	intf
	    	nop
	    	nop
	    	nop
	    	nop
	    	nop
	    	nop
	    	nop
	    	nop
	    	nop
	    	nop
	    	nop
	    	nop	
	bsf	rp0
	bsf	TRISA,1		; Set RA1 as input
	bcf	rp0
	   	return

 
 
;******************************************************************************
; 	this routine checks the busy flag, returns when not busy        
;  	affects:                                                       
;      	Temp - returned with busy/address                          
;******************************************************************************
 
busy_check
		clrwdt			; Clear watchdog
	    	bsf	KeyEnable	; Set for LCD input
	    	bcf     Rs    		; Set lcd for command mode
	    	bsf     R_w   		; Setup to read busy flag
	    	bsf     Disp_En    	; Set En high 
	    	movfw   Lcd_Data     	; Read upper nibble busy flag
	    	movwf   Temp
		bcf     Disp_En    	; 
	    	bsf     Disp_En   	; Toggle En to get other nibble  
        	bcf     Disp_En		;  and discard
	    	btfsc   Temp,7		; Check busy flag, high = busy
	    	goto    busy_check      ;  if busy, check again
	    	bsf	KeyEnable
	    	bsf     rp0    		; Select register page 1
	    	movlw   Portb_Out
	    	andwf   TRISB,f 	; Set for disp output
	   	bcf     rp0    		; Select register page 0
	    	bsf	R_w		; ##########################
		return

LCD_Send
	   	movwf   Temp		; Save data
	   	movlw	h'0f'		; Mask of LCD pins
	   	andwf	Lcd_Data,f     	; send data to lcd
	   	movfw	Temp
	   	iorwf	Lcd_Data,f	; Or data with port
		Return 
